#=======================================================#
#                                                       #
#   Project:    SNA for CJ                              #
#   Author:     Andrew V. Papachristos                  #
#   Copyright:  2015                                    #
#														                            #
#	  Module:     Visualization										        #
#														                            #
#   Tasks:      Setting up RStudio Session				      #
#				        Importing Edgelist and Attribute Data   #
#               Setting Up the Social Network           #  
#               Basic Plot Command                      #
#               Plot Layout                             #
#               Adjusting Vertex (Node) Properties      #
#               Adjusting Edge Properties               #
#               Fixing Node Position                    #
#               Adjusting the Plotting Region           #
#               Saving Work                             #
#               Two-Mode Data & Two-Mode Network        #
#               Plotting Two-Mode Networks              #
#               One-Mode Projections                    #
#               Review                                  #
#                                                       #
#=======================================================#

#--------------------------------------------------------------------

#   1.    Setting up RStudio Session 

# Congratulations on setting your working directory and opening the visualization.R script file.

# Check the contents of the working directory using the dir() command. Printed in the Console pane below will be a list of all the items available in the working directory. You should see the name of this R script as well as the 3 csv files.

dir()

# Open the igraph library. 

library(igraph)





#--------------------------------------------------------------------

#   2.    Importing Edgelist and Attribute Data

# Do you remember the edgelist and attribute file that we worked with in the Module 2: Data Lab? We will be importing those same csv files here. First, load the edgelist using the read.csv() command. Name the RStudio object for this csv "e" for edges.

e <- read.csv("edgelist.csv", header=TRUE)

# Inspect the new object called "e" to check how the csv imported. 

e

# The object "e" contains a header plus 7 rows and 3 columns. The numbers 1-7 to the left of each row are row identifiers and not a separate column. 

# We will want to customize the visualization of nodes in this network, which requires an attribute file. Next, use the read.csv() command to load the attribute data. We will name the RStudio object for this csv "a" for attributes.

a<-read.csv("attributes.csv", header=TRUE)

# Inspect the new object called "a" to check how the csv imported. 

a

# The object "a" contains 6 rows of information plus a header row and 4 columns of information plus a column of numbers identifying each row. We can see that the names in object "a" match the names in object "e".





#--------------------------------------------------------------------

#   3.    Setting Up the Social Network

# Run the command that we learned in Module 2: Data RStudio that creates a network from an edgelist and matches individual-level attribute data to the correct nodes. We will call this new network object "g".

g <- graph.data.frame(e, directed=FALSE, vertices=a)

# Inspect the new network object "g". 

g

# If "g" looks familiar and contains information on nodes (v/c, v/n) and ties (e/n), then we are ready to plot.





#--------------------------------------------------------------------

#   4.    Basic Plot Command

# Below is the basic igraph plotting command briefly introduced in Module 2: Data RStudio. This basic command uses only the default settings in igraph plot. Be sure to select the Plots tab in the lower-right pane to view the results. Click on the Zoom button to see a larger version of any plot.

plot(g)

# Rerun the plot() command several times. Note that with every run the location of the nodes changes. This is because the default in igraph is to place randomly the nodes in the 2-dimensional plotting space. 

# There is a long list of settings that can be adjusted to customize igraph plots. This script walks participants through some of the basic options and iteratively builds upon the basic plot command for a customized result. For a complete list of plotting options type "igraph.plotting" in the search box of the Help pane, see the "drawing graphs" section of the igraph documentation available at (http://cran.r-project.org/web/packages/igraph/igraph.pdf), or run the following command: 

?igraph.plotting





#--------------------------------------------------------------------

#   5.    Plot Layout

# Plot layout refers to the placement of the nodes within the plotting space. The igraph plot() default is random placement. A complete list of options for the "layout=" argument is available with the following command. Click on the "Generate coordinates for plotting graphs" link within the Help pane to see a list of available layouts in igraph.

?layout

# Let's explore just a few of the layout options, to see how they are similar or differ. Run the following commands to see how each layout changes the shape of the network "g".

plot(g, layout=layout.circle)

plot(g, layout=layout.kamada.kawai)

plot(g, layout=layout.fruchterman.reingold)

# The differences between some of the layouts are rather subtle with this small network. Note that clicking on the blue left and right arrow buttons at the top of the Plots pane scrolls through previous versions of plots.





#--------------------------------------------------------------------

#   6.    Adjusting Vertex (Node) Properties

# Run object "a" now to recall the original structure of the attribute data.

a

# Run object "g" now to see the "attr:" (attributes) that we can work with when plotting this network. The "v" stands for "vertices".

g

# Use the $ extraction command to generate a list of each attribute that is stored in the network "g". These match the columns in the attribute list "a". 

V(g)$name
V(g)$gender
V(g)$gpa
V(g)$age

# Now lets use these variables to adjust the appearance of the vertices or nodes. Below are examples of commands that change the default settings of igraph plot(). Notice the different settings in the arguments across the commands. Run each command separately to see the small differences the changes make across the plots.



#   6.1   Vertex Size
# The "vertex.size=" argument changes the size of the node in the plot. The default value for "vertex.size=" is 15. Use the commands below to set the vertex size first to 1 and then to 40. Remember to click on the Zoom button in the Plots pane to see a non-compressed version of your network.

plot(g, layout=layout.kamada.kawai, vertex.size=1)

plot(g, layout=layout.kamada.kawai, vertex.size=40)



#   6.2   Vertex Label
# The "vertex.label=" argument changes the label or text assigned to each node. The default for "vertex.label=" is the vertex id which was generated earlier with the graph.data.frame() command of this script. In this example network the names were the vertex ids pulled from the edgelist. The first command below set the "vertex.label=" to "NA", which removes the vertex labels. The second command below changes the vertex label to equal the numerical grade point average (gpa, one of the attributes of this network), which overrides the vertex id that equals names. Note that the variable for gpa, V(g)$gpa, is the same command used above to inspect the list of the gpa attribute.

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=NA)

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa)



#   6.3   Vertex Shape
# The "vertex.shape=" argument changes the shape of each node. The default for "vertex.shape=" is circle. Note that the name of the shape must be enclosed in quotation marks following the equal sign of the "vertex.shape=" argument. Setting the vertex shape to "none" removes the nodes but still includes the labels.

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="rectangle")

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="none")

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="sphere")

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square")



#   6.4   Vertex Color
# The "vertex.color=" argument changes the color of the nodes. The default for "vertex.color=" is "SkyBlue2". Colors can be quite advanced in R and have descriptive names, numerical identifiers, or numerical adjustments. A simple option is to choose a basic color and type the name between the quotation marks to see if RStudio recognizes that color name. Typing in "Colors in R" into a search engine on your preferred web browser yields multiple results of pdf files displaying R colors and their accompanying names and numbers. See for example: 
#   (http://www.stat.columbia.edu/~tzheng/files/Rcolor.pdf)
#   (http://research.stowers-institute.org/efg/R/Color/Chart/ColorChart.pdf)


# Let's make the nodes green with the command below. Then replace "green" with another color in the command. Be sure to keep the name of the new color between the quotation marks.

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color="green")


# The #CD3278 code stands for for "violetred3". Run the command below to change the color to violet red as designaed by the color code. Then change "#CD3278" to "violetred3" and note that it produces the same vertex color.

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color="#CD3278")   

# Varying the color of nodes by a particular attribute requires an additional attribute variable. For example, to vary the color of the nodes by gender, first generate a new color variable that assigns all nodes the color blue.

V(g)$color <- "blue"

# Then replace the color for only the females to red. Notice that in order to specify just the females as the object to be changed we return to the square brackets introduced in Module 1: Introduction RStudio and we require 2 dollar sign extractions: the first to specify that the column to be changed is color and the second to specify that only rows with gender=="F" should be changed. Another option is to create this color attribute outside of RStudio within your spreadsheet software. You could make a separate column in the attribute csv for color and then reload the attribute file into RStudio.

V(g)$color[V(g)$gender=="F"] <- "red"

# Now instead of assigning just one color to the "vertex.color=" option, we will assign the color attribute variable that we just created. The color variable we created is stored in RStudio as "V(g)$color", so instead of typing "red" or "blue" or the R color numbers for red and blue, we set the color to equal the attribute variable that assigned females the color red and males the color blue.

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color)



#   6.5   Label Color
# The "vertex.label.color=" argument changes the color of the text for each label. The default for "vertex.label.color" is navy blue or black. With the dark red and dark blue nodes, it is difficult to read the label in navy blue or black, so we can use the code below to change the vertex labels to white. 

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white")





#-----------------------------------------------------------------

#   7.    Adjusting Edge Properties

# Run object "e" now to recall the structure of this edgelist.

e

# Remember that each row of this edgelist is a tie connecting the person in the "ego" column to the person in the "alter" column. Because this network is not directed, the reverse is also true. Note that each row also has a value in the "weight" column. Depending on the context of this network this value can take on different meanings. For example, it could be the number of times the individuals saw each other in a given day, the number of times individuals were arrested together, the total dollars exchanged, etc. The graph.data.frame() command ran earlier in this script to create network "g" automatically added in this weight column when it converted the "e" object into network "g". Run "g" now to see "weight (e/n)" as part of this network object. The "e" in "e/n" stands for "edges" and the "n" in "e/n" stands for numeric, meaning that it is a numeric value.

g

# Inspect the list of these weights within the network "g" by extracting the edge variable. Notice that here we switch from the uppercase "V" used above to adjust vertices or nodes to an uppercase "E" to specify the edges.

E(g)$weight

# Note how this list of weights matches the weight column in the edgelist "e".

# This section focuses on commands to adjust the appearance of the lines or edges in a network plot. Some of these commands call in the edge weight variable explored just above. Run each command separately to see the differences that the small changes make across the plots.



#   7.1   Edge Color
# The "edge.color=" argument adjusts the color of the lines in the plot. The default for "edge.color=" is "darkgrey". The command below changes the edge color to "red". Run this command and then change "red" to some other color. Remember to keep the color names in quotation marks. 

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color="red")

# To vary the color of the edges replace the name of one color with the edge weight variable. The numbers in the edge weights correspond to color numbers in RStudio, so the 1s are black and the 2s are red. If you want different colors, you create a new color variable for edges like we did above for vertices. 

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight)



#   7.2   Edge Width
# The "edge.width=" argument adjusts the thickness of the line in the plot. The default for "edge.width" is 1.

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="csquare", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight, edge.width=4)

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="csquare", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight, edge.width=.5)

# To vary the thickness of the edges replace the value with the edge weight variable or create a new variable for edges with your own customization.

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight, edge.width=E(g)$weight)

# If a more dramatic difference is needed between the edge widths, multiply the edge weight variable by a number greater than 1.

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight, edge.width=E(g)$weight*3)



#   7.3   Edge Type
# The "edge.lty=" argument adjusts the type of line in the plot. The default for "edge.lty=" is "solid" or 1. Line types in RStudio have a numerical value (0-6) as well as an accompanying name (0="blank", 1="solid", 2="dashed", 3="dotted", 4="dotdash", 5="longdash", 6="twodash"). Either the value or the name between quotation marks can be used with the "edge.lty" argument. 

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight, edge.width=E(g)$weight*3, edge.lty=3)

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight, edge.width=E(g)$weight*3, edge.lty="dotted")

# To vary the line type of the edges replace the value with the edge weight variable.

plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight, edge.width=E(g)$weight*3, edge.lty=E(g)$weight)





#-----------------------------------------------------------------

#   8.    Fixing Node Position

# Notice how the network moves around the plotting region every time a plot command is run. To fix the position of the nodes, run the set.seed() command with the same number between the parentheses before every plot command. This will start the network from the same place every time. The example below chose the number "12345" as a random value to set the seed. Running this set of commands together multiple times produces the same network image. 

set.seed(12345)
plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight, edge.width=E(g)$weight*3, edge.lty=E(g)$weight)

# If you don't like the arrangement of how this seed fixed the nodes, you can change the random numbers in the parentheses of set.seed().





#-----------------------------------------------------------------

#   9.    Adjusting the Plotting Region

#   9.1   Plotting Region Margins
# Behind the network plot is the white space. This white space is called the graphical region. The graphical region has its own set of default settings, and a different set of arguments are required to adjust any of the settings for the graphical region. The command par(), which is short for parameters, accesses a long list of arguments that can be adjusted for the graphing region. For a complete list of arguments, run the following help command.

?par

# Changing graphical parameters requires running the par() command before the plot() command. Returning to the default parameters requires opening a new graphical device.

# Margins are one of many graphical parameters that can be changed. The "mar=" argument is short for margins. The default value for margins looks like this:  par(mar=(c(5, 4, 4, 2) + 0.1). The list of numbers (a.k.a. the numerical vector) following the lowercase "c" assigns four margin values in the following order (bottom margin, left margin, top margin, right margin), so starting from the bottom the margins are listed clockwise. See "mai" (margins by inch) and "mar" (margins) under the par() (parameters) help menu for more information. To get rid of margins in the graphical space set all four margins to zero. Remember to run all three of the commands together: par(), set.seed(), and plot().

par(mar=c(0,0,0,0))
set.seed(12345)
plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight, edge.width=E(g)$weight*3, edge.lty=E(g)$weight)



#   9.2   Changing the Color of the Plotting Region
# Changing the color of the graphical region also requires an option under graphical parameters, par(). The argument is "bg=" (background) and should be added to the list of par() arguments. The default "bg=" is "white". Note that the command below includes a "+1" after the variable for "edge.color=". This will adjust the color palette by 1 so that black is no longer one of the edge colors that would be hidden on a black background.

par(mar=c(0,0,0,0), bg="black")
set.seed(12345)
plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight+1, edge.width=E(g)$weight*3, edge.lty=E(g)$weight)



#   9.3   Adding Titles to the Plotting Region
# Titles require separate command lines, but they appear within the plotting region. The command is called title() and arguments include "main=" for the main title, "sub=" for the sub-title, "col.main=" for the color of the text of the main title, and "col.sub=" for the color of the text of the sub-title. See the help menu for more options.

?title

# Titles require margin space, so the command below changes the margins from all zeros to adding a bottom and top margin. Running just a title command will layer the title on the most recent graph. 

par(mar=(c(5, 0, 3, 0)), bg="black")
set.seed(12345)
plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight+1, edge.width=E(g)$weight*3, edge.lty=E(g)$weight)
title(main="The Best Network Ever", sub="By A.V. Papachristos", col.main="white", col.sub="lightgrey")



#   9.4   Adding Legends to the Plotting Region
# Much like titles, legends require an additional command and will be added to the top of the most recent plot, legend(). In the legend() command below "topleft" specifies where in the graphing region the legend should be located. The argument "legend=c()" contains a list of text for the labels that appear in the legend. The argument "col=c()" contains a list of the colors that appear in the legend. The argument "pch=" specifies the plotting symbol in the legend. The argument "text.col=" adjusts the color of the text appearing in the legend. The default for legend "text.col=" is "black". The code below resets the parameter margins back to their default settings, so that there is room in the graphing space for the legend. A more complete list of legend arguments are located in legend() help:

?legend

par(mar=(c(5, 0, 3, 0)), bg="black")
set.seed(12345)
plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight+1, edge.width=E(g)$weight*3, edge.lty=E(g)$weight)
title(main="The Best Network Ever", sub="By A.V. Papachristos", col.main="white", col.sub="lightgrey")
legend("topleft", legend=c("male", "female"), col=c("blue", "red"), pch=15, text.col="white")





#-----------------------------------------------------------------

#   10.   Saving Work

#   10.1  Saving Plots
# To save a plot, click on the Export button in the Plots pane, and click "Save Plot as PDF...". I recommend changing the pdf custom size to 10 inches by 10 inches in order to avoid the small compressed version of the plot that appears in the Plots pane. However, this might crop off the legend in your plot. Create a file name in the file name box and save. The plot will be saved in the working directory, which was set to the Visualization RStudio Files folder at the beginning of this session. 

# A more complicated but useful alternative when generating a large number of plots is using the pdf() command in combination with the dev.off() command. Before running the entire set of plot commands add a new beginning pdf() command that opens a pdf graphic device that will hold the next plot generated in RStudio. Immediately after the plotting command, close the graphic device using the dev.off() (graphing device off) command. A pdf of the network should appear in the working directory under the name created in pdf("") between the quotation marks. Below is the same set of plotting commands as above with the addition of the pdf() and dev.off() commands. Run the entire set at once, and then check the working directory folder to see if the plot saved as a pdf file there.

pdf("best_network_ever.pdf")
par(mar=(c(5, 0, 3, 0)), bg="black")
set.seed(12345)
plot(g, layout=layout.kamada.kawai, vertex.size=40, vertex.label=V(g)$gpa, vertex.shape="square", vertex.color=V(g)$color, vertex.label.color="white", edge.color=E(g)$weight+1, edge.width=E(g)$weight*3, edge.lty=E(g)$weight)
title(main="The Best Network Ever", sub="By A.V. Papachristos", col.main="white", col.sub="lightgrey")
legend("topleft", legend=c("male", "female"), col=c("blue", "red"), pch=15, text.col="white")
dev.off()



#   10.2  Saving Scripts
# To save any revisions or additions made to a script file, simply click on the disk icon at the top of the script in this pane. Alternatively, go to Save or Save as under the File menu at the top of the RStudio window.





#-----------------------------------------------------------------

#   11.   Two-Mode Data, Network, & Projections   

# Recall from Module 2: Data RStudio the csv file called "event.csv" based on some hypothetical arrests. This csv contains two-mode data in which individuals are connected through co-arrest events and co-arrest events are connected through shared individuals. Import this csv file using the read.csv() command and remind RStudio to maintain all of the columns as characters rather than numbers using the "colClasses="character"" argument. We will call the new object "t" for two-mode.   

t <- read.csv("event.csv", header=TRUE, colClasses="character")

# Inspect what we just imported by running just the name of the new object or by clicking on the object "t" under Data in the Environment pane.

t

# Notice that there are two columns in object "t": person and event. There are 10 rows in object "t", and each row represents a tie between a person and an event. 

# In order to convert "t" into a network, we must first make the true/false list that designates the two different types of nodes. First, we make two new lists using the unique() command to remove duplicate elements in our two-mode event data.

person <- unique(t$person)
event <- unique(t$event)

# Then we stack these two lists on top of each other for one list of 13 nodes and add a column that assigns true or false to each item in the list--true for persons and false for events. We will call the new list "tf.list".  

tf.list <- data.frame(name=c(event, person), type=c(rep(FALSE, length(event)), rep(TRUE, length(person))), stringsAsFactors=FALSE)

# Now we will make a network out of the event list we called "t" and use the true/false list to designate between the two types of nodes. We use the graph.data.frame() command, and we create a new object called "two.network".

two.network <- graph.data.frame(t, directed=FALSE, vertices=tf.list)

# Inspect the results.

two.network

 



#-----------------------------------------------------------------

#   12.   Plotting Two-Mode Networks    

# In this section of the script we will visualize and customize the two-mode network. We don't have an attribute file for the person and event nodes in this network, so we will build some customization variables within RStudio.                  

# First, let's set the graphing parameters back to the default settings, and inspect the two-mode network using just the igraph plot() defaults.

par(mar=c(c(5, 4, 4, 2) + 0.1), bg="white")
plot(two.network)

# Notice how the plot(two.network) does not distinguish between the two node types. It is difficult to see that this is even a two-mode network. We will focus our customization on differentiating between the two types of nodes. In the edgelist example above, we iteratively customized the plot() command based on a variety of options. For the two-mode network, let's plan out what we want our final network image to look like, so we can think about the different options we will need to customize. I propose that the final network image should have the arrest events be larger boxes of one color and the people be smaller circles of a different color. Because this is a small network, we should also add some labels to the nodes that name the individuals and number the arrests. Given that proposed network, can you imagine what we need to do in RStudio?

# Before we even begin plotting, we are going to create all of our variables to customize our final network. Let's recall how many people and how many events are in our two-mode network. This information is most easily found in our true/false list.

tf.list

# 5 events and 8 people. 

# Generate a new variable within "two.network" called "new.label". Create a list in order makes unique labels for the 5 arrest events and names for the 8 people. Each label will be in quotation marks and separated by commas that go outside of the quotation marks. 

V(two.network)$new.label <- c("arrest 1", "arrest 2", "arrest 3", "arrest 4", "arrest 5", "Willard", "Anna", "Rema", "Rex", "Benny", "Chris", "Jen",  "Ken")

# Generate a new variable within "two.network" called "shape". Create a list of shapes that correspond to the 5 events and the 8 people. 

V(two.network)$shape <- c("square", "square", "square", "square", "square", "circle", "circle", "circle", "circle", "circle", "circle", "circle", "circle")

# Generate a new variable within "two.network" called "color". Create a list of colors that correspond to the 5 events and 8 people.

V(two.network)$color <- c("cornflowerblue", "cornflowerblue", "cornflowerblue", "cornflowerblue", "cornflowerblue", "olivedrab3", "olivedrab3", "olivedrab3", "olivedrab3", "olivedrab3", "olivedrab3", "olivedrab3", "olivedrab3")

# Generate a new variable within "two.network" called "size". Create a list of sizes that correspong to the 5 events and 8 people.

V(two.network)$size <- c(30,30,30,30,30,20,20,20,20,20,20,20,20)

# Notice all of the new variables you created now show up when we inspect the network object.

two.network

# Using some of the same options in the igraph plot() command that were introduced earlier in this script, we can add the new variables to some of the options. There was a bit of trial and error in producing the plot() command below. For example, the labels were a little too big to fit inside of the nodes, so I added the "vertex.label.cex=" argument which adjusts the label size. This is a multiplier, so a value less than 1 will decrease the label size slightly. The default edges in this network were very small and light compared to the nodes, so I increased the edge width to 4 and assigned all edges the color black. Feel free to play around with some of these customization options to get the two-mode network that you think is best.

plot(two.network, vertex.label=V(two.network)$new.label, vertex.label.color="black", vertex.label.cex=.75, vertex.shape=V(two.network)$shape, vertex.color=V(two.network)$color, vertex.size=V(two.network)$size, edge.width=4, edge.color="black")

# Once you get the visualization that you like, you can save it by adding the pdf() and dev.off() commands and add titles.

pdf("best_twomode_network_ever.pdf")
plot(two.network, vertex.label=V(two.network)$new.label, vertex.label.color="black", vertex.label.cex=.75, vertex.shape=V(two.network)$shape, vertex.color=V(two.network)$color, vertex.size=V(two.network)$size, edge.width=4, edge.color="black")
title(main="The Best Two-Mode Network Ever", sub="By Yours Truly", col.main="olivedrab3", col.sub="cornflowerblue")
dev.off()

# Your two-mode network visualization should have saved in your working directory folder as a pdf file. 





#-----------------------------------------------------------------

#   13.    One-Mode Projections 

# Often researchers are interested in the one-mode projections that occur though two-mode networks--how the people are connected to other people through events or how the events are connected to other events through people. This is easy to do in igraph using the bipartite.projection() command. Let's call this new object "p" for projection.

p <- bipartite.projection(two.network)

# Inspect "p" and notice that "p" has two parts because there are two one-mode projections from every two-mode network.

p

# Pull the two separate networks out of "p" and give them their own network object and name.

event.network <- p[[1]]
person.network <- p[[2]]

# Inspect these one-mode projection networks. 

event.network
person.network

# Notice how all of our customization variables are still attached to these one-mode projections. We can inspect them using the extraction dollar sign.

V(event.network)$new.label
V(event.network)$shape
V(event.network)$color
V(event.network)$size

V(person.network)$new.label
V(person.network)$shape
V(person.network)$color
V(person.network)$size

# We can plot these one-mode projections with minor variations to the plot() command we used above to visualize the two-mode network. The commands below are copied and pasted from the command above, but the network name is changed for the network to be plotted and within each variable. 

# Plot the event network from the one-mode projection.

plot(event.network, vertex.label=V(event.network)$new.label, vertex.label.color="black", vertex.label.cex=.75, vertex.shape=V(event.network)$shape, vertex.color=V(event.network)$color, vertex.size=V(event.network)$size, edge.width=4, edge.color="black")

# Plot the person network from the one-mode projection. 

plot(person.network, vertex.label=V(person.network)$new.label, vertex.label.color="black", vertex.label.cex=.75, vertex.shape=V(person.network)$shape, vertex.color=V(person.network)$color, vertex.size=V(person.network)$size, edge.width=4, edge.color="black")


# If you want to save your network visualizations to your working directory and add titles, run the following commands. 

pdf("event_projection.pdf")
plot(event.network, vertex.label=V(event.network)$new.label, vertex.label.color="black", vertex.label.cex=.75, vertex.shape=V(event.network)$shape, vertex.color=V(event.network)$color, vertex.size=V(event.network)$size, edge.width=4, edge.color="black")
title(main="One-Mode Projection Connecting Events", sub="By Yours Truly", col.main="cornflowerblue", col.sub="cornflowerblue")
dev.off()

pdf("person_projection.pdf")
plot(person.network, vertex.label=V(person.network)$new.label, vertex.label.color="black", vertex.label.cex=.75, vertex.shape=V(person.network)$shape, vertex.color=V(person.network)$color, vertex.size=V(person.network)$size, edge.width=4, edge.color="black")
title(main="One-Mode Projection Connecting People", sub="By Yours Truly", col.main="olivedrab3", col.sub="olivedrab3")
dev.off()





#-----------------------------------------------------------------

#   14.   Review

# This Module 3: Visualization RStudio lab was an introduction to some basic customization options in igraph. Participants should now be comfortable making adjustments to the commands to generate their own unique images. One good way to practice visualization is to find a network image online (such as the google image search suggested in the Module 3: Visualization Lab) or in a book and try to replicate the network in RStudio.

# Line breaks in RStudio code are permitted and do not interrupt the command. Line breaks are sometimes useful when organizing and revising long codes. Below is the entire set of graphing commands built iteratively in this script but visually organized with line breaks. Use this format to review the options presented in this script.

# Recall how the pound sign (#) tells RStudio to treat everything after the pound sign as text and not a command. Pound signs are useful devices in long sets of commands like this example when you want to test removing different arguments without deleting them. Insert a pound sign before any of the arguments below and rerun the entire set of commands. Some pound signs might produce error messages if the argument is a requirement of the command, but others will produce the plot with the default setting for that argument.

# Visualizing the edgelist
par(
  mar=(c(5, 0, 3, 0)), 
  bg="black"
)
set.seed(12345)
plot(
  g, 
  layout=layout.kamada.kawai, 
  vertex.size=40, 
  vertex.label=V(g)$gpa, 
  vertex.shape="square", 
  vertex.color=V(g)$color, 
  vertex.label.color="white", 
  edge.color=E(g)$weight+1, 
  edge.width=E(g)$weight*3, 
  edge.lty=E(g)$weight
  )
title(
  main="The Best Network Ever", 
  sub="By A.V. Papachristos", 
  col.main="white", 
  col.sub="lightgrey"
  )
legend(
  "topleft", 
  legend=c("male", "female"), 
  col=c("blue", "red"), 
  pch=15, 
  text.col="white"
  )

# Visualizing the two-mode event network

par(
  mar=c(c(5, 4, 4, 2) + 0.1), 
  bg="white"
  )
plot(
  two.network, 
  vertex.label=V(two.network)$new.label, 
  vertex.label.color="black", 
  vertex.label.cex=.75, 
  vertex.shape=V(two.network)$shape, 
  vertex.color=V(two.network)$color, 
  vertex.size=V(two.network)$size, 
  edge.width=4, 
  edge.color="black"
  )
title(
  main="The Best Two-Mode Network Ever", 
  sub="By Yours Truly", 
  col.main="olivedrab3", 
  col.sub="cornflowerblue"
  )





#-END----------------------------------------------------------------